#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# This file is part of nautilus-pushbullet
#
# Copyright (C) 2014-2016
# Lorenzo Carbonell Cerezo <lorenzo.carbonell.cerezo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gi
try:
    gi.require_version('Nautilus', '3.0')
except Exception as e:
    print(e)
    exit(-1)
from gi.repository import Nautilus as FileManager
from gi.repository import GObject
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GdkPixbuf

import sys
import os
import json
import urllib
from os.path import basename

try:
    sys.path.insert(
        0,
        '/opt/extras.ubuntu.com/pushbullet-commons/share/pushbullet-commons')
    from commons_comun import internet_on, TOKEN_DIR, TOKEN_FILE, \
        TOKEN_PARAMS, ICON
    from pushbulletapi import PushBullet
    from logindialog import LoginDialog
    from tokenizator import Tokenizator
    from progreso import Progreso
    from doitinbackgroundbullet import DoitInBackgroundVarious
    from dialogs import SendToDialog
except Exception as e:
    print(e)
    exit(-1)

APP = 'nautilus-pushbullet'
VERSION = '0.3.0-0extras14.04.0'
SEPARATOR = u'\u2015' * 10

_ = str


def get_files(files_in):
    files = []
    for file_in in files_in:
        file_in = urllib.unquote(file_in.get_uri()[7:])
        if os.path.isfile(file_in):
            files.append(file_in)
    return files

"""
Tools to send files with Pushbullet
"""


class PushbulletMenuProvider(GObject.GObject, FileManager.MenuProvider):
    """Implements the 'Replace in Filenames' extension to the File Manager\
    right-click menu"""

    def __init__(self):
        """File Manager crashes if a plugin doesn't implement the __init__\
        method"""
        self.pushBulletService = PushBullet()
        self.is_login = self.pushBulletService.is_login()

    def about(self, menu):
        """Create and populate the about dialog."""
        about_dialog = Gtk.AboutDialog()
        about_dialog.set_name(APP)
        about_dialog.set_version(VERSION)
        about_dialog.set_copyright('''
Copyrignt (c) 2014-2016
Lorenzo Carbonell Cerezo
''')
        about_dialog.set_comments(_('An indicator for PushBullet'))
        about_dialog.set_license('''
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
this program. If not, see <http://www.gnu.org/licenses/>.
''')
        about_dialog.set_website('http://www.atareao.es')
        about_dialog.set_website_label('http://www.atareao.es')
        about_dialog.set_authors([
            'Lorenzo Carbonell <https://launchpad.net/~lorenzo-carbonell>'])
        about_dialog.set_documenters([
            'Lorenzo Carbonell <https://launchpad.net/~lorenzo-carbonell>'])
        about_dialog.set_translator_credits('''
'Lorenzo Carbonell <https://launchpad.net/~lorenzo-carbonell>\n
''')
        about_dialog.set_icon(GdkPixbuf.Pixbuf.new_from_file(ICON))
        about_dialog.set_logo(GdkPixbuf.Pixbuf.new_from_file(ICON))
        about_dialog.set_program_name(APP)
        about_dialog.run()
        about_dialog.destroy()

    def on_init_clicked(self, widget):
        try:
            self.pushBulletService = PushBullet()
            while(self.pushBulletService.is_login() is False and
                  internet_on()):
                ld = LoginDialog()
                ld.run()
                oauth_access_token = ld.code
                atokenizator = Tokenizator(
                    config_file_dir=TOKEN_DIR,
                    config_file=TOKEN_FILE,
                    default_params=TOKEN_PARAMS)
                atokenizator.set('oauth_access_token', oauth_access_token)
                atokenizator.save()
                self.pushBulletService.restart()
            self.is_login = self.pushBulletService.is_login()
        except Exception as e:
            print(e)

    def on_send_push_clicked(self, widget, selected):
        try:
            if self.is_login is True and internet_on() is True:
                devices = self.pushBulletService.getDevices()
                contacts = self.pushBulletService.getContacts()
                tdevices = []
                tcontacts = []
                for device in devices:
                    if 'nickname' in devices[device].keys() and\
                            len(devices[device]['nickname']) > 0:
                        label = devices[device]['nickname']
                        tdevices.append(
                            {'device_iden': device, 'label': label})
                    elif 'model' in devices[device].keys() and\
                            len(devices[device]['model']) > 0:
                        label = devices[device]['model']
                        tdevices.append(
                            {'device_iden': device, 'label': label})
                sdevices = sorted(tdevices, key=lambda k: k['label'])
                for contact in contacts:
                    if contacts[contact]['active'] and\
                            'name' in contacts[contact].keys() and\
                            'email' in contacts[contact].keys():
                        tcontacts.append({
                            'contact_iden': contact,
                            'name': contacts[contact]['name'],
                            'email': contacts[contact]['email']})
                scontacts = sorted(tcontacts, key=lambda k: k['name'])
                sdevices = sorted(tdevices, key=lambda k: k['label'])
                cm = SendToDialog(sdevices, scontacts)
                if cm.run() == Gtk.ResponseType.ACCEPT:
                    cm.hide()
                    ans = cm.get_response()
                    label = ans['label']
                    iden = ans['iden']
                    ttype = ans['type']
                    files = get_files(selected)
                    progreso = Progreso(_('Pushing files'), None, len(files))
                    dibv = DoitInBackgroundVarious(
                        self.pushBulletService.send_a_file,
                        files,
                        iden, '', ttype)
                    progreso.connect('i-want-stop', dibv.stop_it)
                    dibv.connect('start_one', progreso.set_todo_label)
                    dibv.connect('finished_one', progreso.increase)
                    dibv.connect('interrupted', progreso.close)
                    dibv.connect('finished', progreso.close)
                    dibv.start()
                    progreso.run()
                cm.destroy()
        except Exception as e:
            print(e)

    def get_file_items(self, window, sel_items):
        """Adds the 'Replace in Filenames' menu item to the File Manager
           right-click menu, connects its 'activate' signal to the 'run'
           method passing the selected Directory/File"""
        top_menuitem = FileManager.MenuItem(
            name='PushbulletMenuProvider::Gtk-pdf-tools',
            label=_('Pushbullet'),
            tip=_('Send files with Pushbullet'),
            icon='Gtk-find-and-replace')
        #
        submenu = FileManager.Menu()
        top_menuitem.set_submenu(submenu)
        try:
            if self.pushBulletService is not None and\
                    self.is_login is True:
                sub_menuitem_95 = FileManager.MenuItem(
                    name='PushbulletMenuProvider::Gtk-pushbullet-95',
                    label=_('Send to ...'),
                    tip=_('Send to ...'),
                    icon='Gtk-find-and-replace')
                sub_menuitem_95.connect(
                    'activate',
                    self.on_send_push_clicked,
                    sel_items)
                submenu.append_item(sub_menuitem_95)
            else:
                sub_menuitem_97 = FileManager.MenuItem(
                    name='PushbulletMenuProvider::Gtk-pushbullet-97',
                    label=_('Init Pushbullet'),
                    tip=_('Init Pushbullet'),
                    icon='Gtk-find-and-replace')
                sub_menuitem_97.connect(
                    'activate',
                    self.on_init_clicked)
                submenu.append_item(sub_menuitem_97)
        except Exception as e:
            print(e)
        #
        sub_menuitem_98 = FileManager.MenuItem(
            name='PushbulletMenuProvider::Gtk-None-02',
            label=SEPARATOR)
        submenu.append_item(sub_menuitem_98)
        #
        sub_menuitem_99 = FileManager.MenuItem(
            name='PushbulletMenuProvider::Gtk-pushbullet-99',
            label=_('About'),
            tip=_('About'),
            icon='Gtk-find-and-replace')
        sub_menuitem_99.connect('activate', self.about)
        submenu.append_item(sub_menuitem_99)
        #
        return top_menuitem,

if __name__ == '__main__':
    pbmp = PushbulletMenuProvider()
    print(pbmp.get_file_items(None, []))
    pass
